/*--------------------------------------------------------------------------*/
/*                                                                          */
/*                                                                          */
/*      ------------         Bit-Bucket Software <no-Inc>                   */
/*      \ 10001101 /         Writers and Distributors of                    */
/*       \ 011110 /          No-Cost<no-tm> Software.                       */
/*        \ 1011 /                                                          */
/*         ------                                                           */
/*                                                                          */
/*  Copyright (C) 1987, 1988, 1989 by Robert Hartman and Vincent Perriello  */
/*                                                                          */
/*                                                                          */
/*               This module was written by Vince Perriello                 */
/*                                                                          */
/*                                                                          */
/*                 BinkleyTerm Nodelist processing module                   */
/*                                                                          */
/*                                                                          */
/*    For complete  details  of the licensing restrictions, please refer    */
/*    to the License  agreement,  which  is published in its entirety in    */
/*    the MAKEFILE and BT.C, and also contained in the file LICENSE.210.    */
/*                                                                          */
/*    USE  OF THIS FILE IS SUBJECT TO THE  RESTRICTIONS CONTAINED IN THE    */
/*    BINKLEYTERM  LICENSING  AGREEMENT.  IF YOU DO NOT FIND THE TEXT OF    */
/*    THIS  AGREEMENT IN ANY OF THE  AFOREMENTIONED FILES,  OR IF YOU DO    */
/*    NOT HAVE THESE FILES,  YOU SHOULD  IMMEDIATELY CONTACT THE AUTHORS    */
/*    AT THE  ADDRESSES LISTED BELOW.  IN NO EVENT SHOULD YOU PROCEED TO    */
/*    USE   THIS  FILE  WITHOUT  HAVING   ACCEPTED  THE  TERMS  OF   THE    */
/*    BINKLEYTERM  LICENSING AGREEMENT,  OR SUCH OTHER  AGREEMENT AS YOU    */
/*    ARE ABLE TO REACH WITH THE AUTHORS.                                   */
/*                                                                          */
/*                                                                          */
/*    The Authors can be reached at the following addresses:                */
/*                                                                          */
/*    Robert C. Hartman                      Vincent E. Perriello           */
/*    Spark Software                         VEP Software                   */
/*    427-3 Amherst Street                   111 Carroll Street             */
/*    CS2032, Suite 232                      Naugatuck, CT 06770            */
/*    Nashua, NH 03061                                                      */
/*                                                                          */
/*    FidoNet 1:132/101                      FidoNet 1:141/491              */
/*    Data    (603) 888-8179                 Data    (203) 729-7569         */
/*                                                                          */
/*    Please feel free to contact us at any time to share your comments     */
/*    about our software and/or licensing policies.                         */
/*                                                                          */
/*--------------------------------------------------------------------------*/

#include <sys\types.h>
#include <sys\stat.h>
#include <stdio.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <conio.h>
#include <dos.h>
#include <io.h>
#include <string.h>

#ifdef __TURBOC__
#include <alloc.h>
#define _fmalloc(n) farmalloc (n)
#else
#include <malloc.h>
#endif

#include "com.h"
#include "xfer.h"
#include "zmodem.h"
#include "keybd.h"
#include "sbuf.h"
#include "sched.h"
#include "externs.h"
#include "prototyp.h"

#ifdef OS_2  /*PLF Fri  05-05-1989  08:50:14 */
#define INCL_DOS
#include <os2.h>
#define _dos_read(fh, buf, size, sizep) DosRead(fh, buf, size, (PUSHORT)sizep)
#endif

static char far *get_size (unsigned int);
static int get_new_info (unsigned);
static int get_old_info (unsigned);
static int get_TBBS_info (unsigned, int, int);
static int Quick_Next_Zone (void);
static int opus_next_zone (void);

static unsigned int idx_size = 0;                /* number of entries in it   */
static unsigned int zone_offset = 0;
static int last_zone = -1;
static int extra_bytes = 0;

static char far *get_size (n)
unsigned int n;
{
   /* If we get this far, then we have to use a straight far pointer */
   return (_fmalloc (n));
}

/*---------------------------------------------------------------------------*/
/* NODEPROC                                                                  */
/* Find nodelist entry and set baud to nodelist baud for dialing out         */
/*---------------------------------------------------------------------------*/

int nodeproc (nodeaddr)
char *nodeaddr;
{
   int opuszone, opusnet, opusnode;              /* zone, net and node */
   char *c, *skip_blanks ();

   c = skip_blanks (nodeaddr);                   /* get rid of the blanks     */
   if (sscanf (c, "%d:%d/%d", &opuszone, &opusnet, &opusnode) != 3)
      {
      opuszone = alias[0].Zone;
      if (sscanf (c, "%d/%d", &opusnet, &opusnode) != 2)
         {
/*
         opusnet = alias[0].Net;
         if (sscanf (c, "%d", &opusnode) != 1)
            {
            status_line ("!Can't convert Net/Node address");
            return (0);
            }
*/
status_line ("!Can't convert Net/Node address");    /*PLF Mon  05-08-1989  07:26:29 */
return (0);
         }
      }
   if (!nodefind (opuszone, opusnet, opusnode, 1)){ /* if we can't find the node */
      status_line ("!Can't convert Net/Node address");    /*PLF Mon  05-08-1989  07:26:29 */
      return (0);                                /* go away now               */
    }
   if (opuszone != alias[0].Zone)
      {
      status_line ("*Processing node %d:%d/%d -- %s", opuszone, opusnet, opusnode, newnodedes.SystemName);
      }
   else
      {
      status_line ("*Processing node %d/%d -- %s", opusnet, opusnode, newnodedes.SystemName);
      }
   if (!CARRIER)                                 /* if no carrier yet,        */
      {
      if (autobaud)
         set_baud (max_baud, 1);             /* Set to our highest baud
                                                  * rate */
      else set_baud ((300 * newnodedes.BaudRate), 1);       /* set baud to nodelist
                                                                 * baud */
      }
   return (1);                                   /* return success to caller  */
}

/*---------------------------------------------------------------------------*/
/* NODEFIND                                                                  */
/* Find nodelist entry for use by other routines (password, nodeproc)        */
/* If found, result will be in "newnodedes".                                 */
/*---------------------------------------------------------------------------*/

int nodefind (binkzone, binknet, binknode, prtflag)
int binkzone, binknet, binknode;                 /* zone, net and node */
int prtflag;
{
   int i, j;
   int have_boss_data = 0;
   int need_boss_data = 0;

   newnodedes.NetNumber = newnodedes.NodeNumber = found_zone = 0;

   CurrentOKFile = DEFAULT.rq_OKFile;    /* Set the default f.req paths */
   CurrentFILES = DEFAULT.rq_FILES;
   CurrentAbout = DEFAULT.rq_About;
   CurrentReqTemplate = DEFAULT.rq_Template;
   CurrentNetFiles = DEFAULT.sc_Inbound;
   CurrentReqLim = DEFAULT.rq_Limit;

   if ((binknet == boss_net) && (binknode == boss_node) &&
       (binkzone == alias[0].Zone))
      {
      ++need_boss_data;
    /*PLF Mon  05-08-1989  06:07:21 */
    /* BugAmundo. Referencing NULL pointers causes a GP Fault in os/2
     *if (strlen (BOSSphone) > 2)
     *   ++have_boss_data;
     *if (strlen (BOSSpwd) > 2)
     *   ++have_boss_data;
     *}
     */
      if (BOSSphone && strlen (BOSSphone) > 2)
         ++have_boss_data;
      if (BOSSpwd && strlen (BOSSpwd) > 2)
         ++have_boss_data;
      }

   if (!binkzone)
      {
      i = (*nodefunc) (alias[0].Zone, binknet, binknode, have_boss_data);
      if (i) goto lookup_done;
      }
   i = (*nodefunc) (binkzone, binknet, binknode, have_boss_data);

lookup_done:

   assumed = 0;                              /* Default to zone of first */
   for (j = 0; j < num_addrs; j++)
      {
      if (alias[j].Zone == found_zone)
         {
         assumed = j;
         break;
         }
      }
      
   if (!i && (have_boss_data != 2))
      {
      if (prtflag)
         status_line ("!Couldn't find Zone/Net/Node: %d:%d/%d",
                      binkzone, binknet, binknode);

      if (curmudgeon && CARRIER && binknet != -1 && binknode != -1)
         {
         status_line ("!Disconnecting nuisance caller");
         DTR_OFF ();                            /* Hang up right now      */
         timer (2);                             /* Wait two secs          */
         DTR_ON ();                             /* OK, turn modem back on */
         }

      }

   if (binkzone == -1)
      return i;

   /* If we found the entry, then we promote the file request
    * to the "KNOWN" class. If the password field is non-zero,
    * then promote to "PROT". It's OK to do that since the higher
    * level code will hang up before f.req's if the password does
    * not match.
    *
    */

   if (i)
      {
      if (newnodedes.Password[0])
         {
         CurrentOKFile = PROT.rq_OKFile;
         CurrentFILES = PROT.rq_FILES;
         CurrentAbout = PROT.rq_About;
         CurrentReqTemplate = PROT.rq_Template;
         CurrentNetFiles = PROT.sc_Inbound;
         CurrentReqLim = PROT.rq_Limit;
         }
      else
         {
         CurrentOKFile = KNOWN.rq_OKFile;
         CurrentFILES = KNOWN.rq_FILES;
         CurrentAbout = KNOWN.rq_About;
         CurrentReqTemplate = KNOWN.rq_Template;
         CurrentNetFiles = KNOWN.sc_Inbound;
         CurrentReqLim = KNOWN.rq_Limit;
         }
      }

   if (!need_boss_data)
      return (i);

/*
 *	We can get here one of two ways:
 *
 *	1) No nodelist data was found and this is the BOSS.
 *
 *	2) Nodelist lookup occurred, but this is the BOSS.
 *
 *	For case (1), have_boss_data MUST be 2 (meaning we have
 *	both a phone number and a password entry). If that is the
 *	case, fill in newnodedes with mostly zeroes, plugging in
 *	the BOSS net, node, phone number and password.
 *
 *	For case (2), just see if there is any substitution for
 *	BOSSphone and/or BOSSpwd, then exit.
 *
 */

   if (BOSSphone && strlen (BOSSphone) > 2) /*PLF Mon  05-08-1989  06:09:41 */
      {
      strncpy (newnodedes.PhoneNumber, BOSSphone, 40);
      newnodedes.PhoneNumber[39] = '\0';
      }

   if (BOSSpwd && strlen (BOSSpwd) > 2) /*PLF Mon  05-08-1989  06:09:51 */
      {
      strncpy (newnodedes.Password, BOSSpwd, 8);
      newnodedes.Password[7] = '\0';
      }

   if (i)
      return (1);

   /* No BOSS in the nodelist */
   if (have_boss_data != 2){
      status_line ("!No BOSS in the nodelist");    /*PLF Mon  05-08-1989  07:26:29 */
      return (0);
    }

   newnodedes.NodeNumber = binknode;             /* Node Number */
   newnodedes.NetNumber = binknet;               /* Net Number  */
   newnodedes.Cost = newnodedes.RealCost = 0;    /* Assume boss is free */
   strcpy (newnodedes.SystemName, "Binkley's Boss");    /* System Name defaults */
   strcpy (newnodedes.MiscInfo, "Somewhere out There"); /* As does City */
   newnodedes.HubNode = 0;                       /* Don't know who is HUB */
   newnodedes.BaudRate = (char) (max_baud / 300);/* Assume boss speed = ours */
   newnodedes.ModemType = 0;                     /* Or modem type */
   newnodedes.NodeFlags = B_CM;                  /* Assume boss is CM */
   newnodedes.NodeFiller = 0;                    /* Zero out filler */
   return (1);
}

static int Quick_Next_Zone ()
{
   register struct QuickNodeIdxRecord far *nodeidx;     /* index file         */
   register unsigned int i;

   if (no_zones)
      return (-1);

   nodeidx = (struct QuickNodeIdxRecord far *) (node_index + (zone_offset *
              sizeof (struct QuickNodeIdxRecord)));
   for (i = zone_offset; i < idx_size; ++nodeidx, i++)
      {
      if (nodeidx->QI_Zone != last_zone)
         {
         last_zone = nodeidx->QI_Zone;
         zone_offset = i + 1;
         return (last_zone);
         }
      }

   last_zone = -1;
   zone_offset = 0;
   return (-1);
}


int QuickLookup (Quickzone, Quicknet, Quicknode, have_boss_data)
int Quickzone, Quicknet, Quicknode;              /* zone, net and node */
int have_boss_data;                              /* BOSS data flag     */
{
   register struct QuickNodeIdxRecord far *nodeidx;     /* index file         */
   struct QuickNodeListRecord nodedes;           /* desc. of node      */

   int foundnet = 0;                             /* 'found the net' flag      */
   int found = 0;                                /* 'found the node' flag     */
   int idxrec = 0;                               /* record in QNL_IDX.BBS     */
   long nodeoff = 0L;                            /* offset into QNL_DAT.BBS   */
   char temp[80];                                /* where we build filenames  */
   unsigned int i;
   int stream;
   FILE *stream1;
   int got;
   struct stat f;

   newnodedes.NetNumber = newnodedes.NodeNumber = 0;

   if (node_index == NULL)
      {
      temp[0] = '\0';                            /* "null-terminated string"  */
      strcpy (temp, net_info);                   /* take nodelist path        */
      strcat (temp, "QNL_IDX.BBS");              /* add in the file name      */
      if ((stream = open(temp, O_RDONLY)) == -1)
         {
         if (have_boss_data != 2)
            status_line ("!Unable to open %s", temp);
         return (0);                             /* no file, no work to do    */
         }
      fstat (stream, &f);                        /* get file statistics       */
      i = (unsigned int) f.st_size;              /* size of index file,       */
      idx_size = i / sizeof (*nodeidx);          /* number of index entries   */
      node_index = get_size (i);
      if (node_index == NULL)
         {
         status_line ("!Unable to allocate memory for Nodelist Index");
         close (stream);
         return (0);
         }
      if (_dos_read (stream, node_index, i, &got) != 0)
         {
         status_line ("!Failed to read nodelist index into memory");
         close (stream);
         return (0);
         }
      close (stream);
      }
   nodeidx = (struct QuickNodeIdxRecord far *) node_index;


   if (Quickzone < 0)
      {
      return (Quick_Next_Zone ());
      }
   else if (no_zones)
      Quickzone = 0;

   for (i = 1; i <= idx_size; idxrec++, nodeidx++, i++)
      {
      if (((Quickzone == nodeidx->QI_Zone) || (Quickzone == 0))
          && (Quicknet == nodeidx->QI_Net))
         {
         foundnet = 1;                           /* say we found the net      */
         if (((Quicknode == 0) && (nodeidx->QI_Node <= 0))
             || (nodeidx->QI_Node == Quicknode)) /* see if we found the node  */
            {
            found = 1;                           /* say we found it           */
            break;                               /* get out                   */
            }
         }
      else if (foundnet)                         /* already past the net?     */
         break;                                  /* Yes, we failed...         */
      }

   if (!found)
      {
      return (0);
      }


   nodeoff = (long) idxrec *(long) sizeof (nodedes);    /* actual file offset   */

   strcpy (temp, net_info);                      /* take nodelist path        */
   strcat (temp, "QNL_DAT.BBS");                 /* add in the file name      */
   if ((stream1 = fopen (temp, "rb")) == NULL)   /* OK, let's open the file   */
      {
      status_line ("!Unable to open %s", temp);
      return (0);
      }

   if (fseek (stream1, nodeoff, SEEK_SET))       /* try to point at record    */
      {
      status_line ("!Unable to position to node record in %s", temp);
      fclose (stream1);
      return (0);
      }

   if (!fread (&nodedes, sizeof (nodedes), 1, stream1))
      {
      status_line ("!Not able to read node record from %s", temp);
      fclose (stream1);
      return (0);
      }
   fclose (stream1);

   /*
    * Copy data from nodedes into newnodedes. 
    */

   newnodedes.NodeNumber = nodedes.QL_Node;      /* Node Number  */
   newnodedes.NetNumber = nodedes.QL_Net;        /* Net Number   */
   newnodedes.Cost = nodedes.QL_Cost;            /* Cost         */

   i = min (nodedes.QL_Name[0], 19);
   strncpy (&newnodedes.SystemName[0], &nodedes.QL_Name[1], i);
   newnodedes.SystemName[i] = '\0';              /* System Name  */

   i = min (nodedes.QL_Phone[0], 39);
   strncpy (&newnodedes.PhoneNumber[0], &nodedes.QL_Phone[1], i);
   newnodedes.PhoneNumber[i] = '\0';             /* Phone Number */

   i = min (nodedes.QL_City[0], 29);
   strncpy (&newnodedes.MiscInfo[0], &nodedes.QL_City[1], i);
   newnodedes.MiscInfo[i] = '\0';

   /* This field is not necessarily null terminated */
   i = min (nodedes.QL_Password[0], 8);
   strncpy (&newnodedes.Password[0], &nodedes.QL_Password[1], i);
   if (i < 8)
      newnodedes.Password[i] = '\0';

   /* Adam Hudson now gives us this, so we might as well use it */
   newnodedes.NodeFlags = nodedes.QL_Flags;

   /* Since we have the stuff we need! */
   newnodelist = 1;                              /* We have all the
                                                  * information! */

   newnodedes.RealCost = nodedes.QL_Cost;        /* Cost               */
   newnodedes.HubNode = 0;                       /* Don't know who is Hub    */
   newnodedes.BaudRate = (char) (nodedes.QL_BaudRate / 300);    /* Baud     */
   newnodedes.ModemType = 0;                     /* Don't know modem type    */
   newnodedes.NodeFiller = 0;                    /* Filler should be zero    */
   found_zone = nodeidx -> QI_Zone;              /* Keep track of found zone */
   return (1);
}

static int opus_next_zone ()
{
   register struct _ndi far *nodeidx;            /* index file                */
   register unsigned int i;

   if (no_zones)
      return (-1);

   if (zone_offset == 0)
      zone_offset = 1;

   nodeidx = (struct _ndi far *) (node_index + (zone_offset *
             sizeof (struct _ndi)));

   for (i = zone_offset; i < idx_size; ++nodeidx, i++)
      {
      if (nodeidx->node == -2)
         {
         last_zone = nodeidx->net;
         zone_offset = i + 1;
         return (last_zone);
         }
      }

   last_zone = -1;
   zone_offset = 0;
   return (-1);
}

int opusfind (opuszone, opusnet, opusnode, have_boss_data)
int opuszone, opusnet, opusnode;                 /* zone, net and node */
int have_boss_data;                              /* BOSS data flag     */
{
   register struct _ndi far *nodeidx;            /* index file                */
/* int foundnet = 0;   */                        /* 'found the net' flag      */
   int found = 0;                                /* 'found the node' flag     */
   int nodeoff = 0;                              /* offset into nodelist.sys  */
   char temp[80];                                /* where we build filenames  */
   unsigned int i;
   int stream;
   struct stat f;
   int current_zone = 0;
   int got;

   newnodedes.NetNumber = newnodedes.NodeNumber = 0;

   if (node_index == NULL)
      {
      temp[0] = '\0';                            /* "null-terminated string"  */
      strcpy (temp, net_info);                   /* take nodelist path        */
      strcat (temp, "NODELIST.IDX");             /* add in the file name      */
      if ((stream = open(temp, O_RDONLY)) == -1)
         {
         i = 0;                                  /* Need this later           */
         if (have_boss_data != 2)
            status_line ("!Unable to open Nodelist.Idx file");
         return (0);                             /* no file, no work to do    */
         }
      fstat (stream, &f);                        /* get file statistics       */
      i = (unsigned int) f.st_size;              /* size of index file,       */
      idx_size = i / sizeof (struct _ndi);       /* number of index entries   */

      /*
       * Now take into account that the .DAT file can be bigger than we
       * really expect it to be.  Just take the number of records, and
       * divide into the size of the .DAT file to find the true record size
       */
      if (newnodelist)
         {
         temp[0] = '\0';                            /* "null-terminated string"  */
         strcpy (temp, net_info);                   /* take nodelist path        */
         strcat (temp, "NODELIST.DAT");             /* add in the file name      */
         if (!stat (temp, &f))
            {
            extra_bytes = ((int) (f.st_size / idx_size)) - sizeof (newnodedes);
            }
         }

      node_index = get_size (i);
      if (node_index == NULL)
         {
         status_line ("!Unable to allocate memory for Nodelist Index");
         close (stream);
         return (0);
         }
      if (_dos_read (stream, node_index, i, &got) != 0)
         {
         status_line ("!Failed to read nodelist index into memory");
         close (stream);
         return (0);
         }
      close (stream);
      }

   nodeidx = (struct _ndi far *) node_index;

   /* If using the old nodelist, bypass zone stuff - it isn't there */
   if (opuszone < 0)
      {
      return (opus_next_zone ());
      }
   else if ((!newnodelist) || (alias[0].Zone == 0) || (no_zones))
      {
      opuszone = 0;
      }

   for (i = 1; i <= idx_size; nodeoff++, nodeidx++, i++)
      {
      if (nodeidx->node == -2)
	 current_zone = nodeidx->net;
	 
      if (opuszone > 0 && current_zone != opuszone)
            continue;

      if (nodeidx->net == opusnet)               /* if a match on net,        */
         {
/*       foundnet = 1;   */                      /* say we found the net      */
         if (((opusnode == 0) && (nodeidx->node <= 0))
             || (nodeidx->node == opusnode))     /* see if we found the node  */
            {
            found = 1;                           /* say we found it           */
            break;                               /* get out                   */
            }
         }

      /*
       * This code is good, but many people complained about not being able
       * to add nodes to nets by adding them at the end of the nodelist.  So,
       * I commented this stuff out for now. - Bob
       */
/*      else*/
      /*      if (foundnet)*//* already past the net?     */
      /*         break;*//* Yes, we failed...         */
      }

   if (!found)
      return (0);

   found_zone = current_zone;                    /* Keep track of found zone */

   if (newnodelist)
      i = (get_new_info (nodeoff));
   else i = (get_old_info (nodeoff));

   return (i);
}


static int get_old_info (recno)
unsigned recno;
{
   struct _node nodedes;                         /* desc. of node             */
   long nodeoff;                                 /* Offset into NODELIST.SYS  */
   char temp[80];                                /* where we build filenames  */
   char *c, *ch;
   int i;
   FILE *stream;

   nodeoff = (long) recno *(long) sizeof (nodedes);     /* actual file offset        */

   strcpy (temp, net_info);                      /* take nodelist path        */
   strcat (temp, "NODELIST.SYS");                /* add in the file name      */
   if ((stream = fopen (temp, "rb")) == NULL)    /* OK, let's open the file   */
      {
      status_line ("!Unable to open Nodelist.Sys file");
      return (0);
      }

   if (fseek (stream, nodeoff, SEEK_SET))        /* try to point at record    */
      {
      status_line ("!Unable to position to node record in Nodelist.Sys");
      fclose (stream);
      return (0);
      }

   if (!fread (&nodedes, sizeof (nodedes), 1, stream))
      {
      status_line ("!Not able to read node record from Nodelist.Sys");
      fclose (stream);
      return (0);
      }
   fclose (stream);

   /*
    * Copy data from nodedes into newnodedes. 
    */

   newnodedes.NodeNumber = nodedes.number;       /* Node Number  */
   newnodedes.NetNumber = nodedes.net;           /* Net Number   */
   newnodedes.Cost = nodedes.cost;               /* Cost         */

   strncpy (&newnodedes.SystemName[0], &nodedes.name[0], 20);
   newnodedes.SystemName[19] = '\0';             /* System Name  */

   strncpy (&newnodedes.PhoneNumber[0], &nodedes.phone[0], 40);
   newnodedes.PhoneNumber[39] = '\0';            /* Phone Number */

   strncpy (&newnodedes.MiscInfo[0], &nodedes.city[0], 30);
   newnodedes.MiscInfo[29] = '\0';

   ch = NULL;                                    /* Pointer to password      */
   c = nodedes.city;                             /* Point to start of city   */
   i = 37;                                       /* No password if this = 0  */
   while (i--)                                   /* Enforce that limit       */
      {
      if (*c++ != '\0')                          /* If not at end of city,   */
         continue;                               /* go on to next character  */
      if (*c++ != '!')                           /* End of city, got '!' ??  */
         break;                                  /* No, treat like a failure */
      ch = c;                                    /* Got it, point to it      */
      break;                                     /* Exit with success code   */
      }
   if (ch != NULL)
      {
      strncpy (&newnodedes.Password[0], ch, 8);  /* Copy the password        */
      }
   else newnodedes.Password[0] = '\0';           /* Else set zero length     */

   newnodedes.RealCost = nodedes.cost;           /* Cost                     */
   newnodedes.HubNode = 0;                       /* Don't know who is Hub    */
   newnodedes.BaudRate = (char) (nodedes.rate / 300);   /* Baud Rate                */
   newnodedes.ModemType = 0;                     /* Don't know modem type    */
   newnodedes.NodeFlags = 0;                     /* Don't know any flags     */
   newnodedes.NodeFiller = 0;                    /* Filler should be zero    */
   return (1);
}

static int get_new_info (recno)
unsigned recno;
{
   long nodeoff;                                 /* Offset into NODELIST.DAT  */
   char temp[80];                                /* where we build filenames  */
   FILE *stream;

   /* actual file offset */
   nodeoff = (long) recno * ((long) (sizeof (newnodedes) + extra_bytes));

   strcpy (temp, net_info);                      /* take nodelist path        */
   strcat (temp, "NODELIST.DAT");                /* add in the file name      */
   if ((stream = fopen (temp, "rb")) == NULL)    /* OK, let's open the file   */
      {
      status_line ("!Unable to open Nodelist.Dat file");
      return (0);
      }

   if (fseek (stream, nodeoff, SEEK_SET))        /* try to point at record    */
      {
      status_line ("!Unable to position to node record in Nodelist.Dat");
      fclose (stream);
      return (0);
      }

   if (!fread (&newnodedes, sizeof (newnodedes), 1, stream))
      {
      status_line ("!Not able to read node record from Nodelist.Dat");
      fclose (stream);
      return (0);
      }
   fclose (stream);
   return (1);
}

static int get_TBBS_info (recno, TBBSnet, TBBSnode)
unsigned recno;
int TBBSnet, TBBSnode;
{
   struct nodels nodedes;                        /* desc. of node             */
   struct extrastuff ext;                        /* Extra stuff for Binkley   */
   long nodeoff;                                 /* Offset into NODELIST.DOG  */
   char temp[80];                                /* where we build filenames  */
   FILE *stream;

   --recno;                                      /* TBBS list starts at first
                                                  * record - no filler */

   nodeoff = (long) recno *(long) sizeof (nodedes);     /* actual file offset        */

   strcpy (temp, net_info);                      /* take nodelist path        */
   strcat (temp, "NODELIST.DOG");                /* add in the file name      */
   if ((stream = fopen (temp, "rb")) == NULL)    /* OK, let's open the file   */
      {
      status_line ("!Unable to open Nodelist.Dog file");
      return (0);
      }

   if (fseek (stream, nodeoff, SEEK_SET))        /* try to point at record    */
      {
      status_line ("!Unable to position to node record in Nodelist.Dog");
      fclose (stream);
      return (0);
      }

   if (!fread (&nodedes, sizeof (nodedes), 1, stream))
      {
      status_line ("!Not able to read node record from Nodelist.Dog");
      fclose (stream);
      return (0);
      }
   fclose (stream);

   /*
    * Copy data from nodedes into newnodedes. 
    */

   newnodedes.NodeNumber = TBBSnode;             /* Node Number  */
   newnodedes.NetNumber = TBBSnet;               /* Net Number   */
   newnodedes.Cost = nodedes.nodecost;           /* Cost         */

   strncpy (&newnodedes.SystemName[0], &nodedes.nodename[0], 14);
   newnodedes.SystemName[19] = '\0';             /* System Name  */

   strncpy (&newnodedes.PhoneNumber[0], &nodedes.nodephone[0], 40);
   newnodedes.PhoneNumber[39] = '\0';            /* Phone Number */

   strncpy (&newnodedes.MiscInfo[0], &nodedes.nodecity[0], 30);
   newnodedes.MiscInfo[29] = '\0';

   newnodedes.Password[0] = '\0';                /* Else set zero length     */

   newnodedes.RealCost = nodedes.nodecost;       /* Cost                     */
   newnodedes.HubNode = nodedes.nodehub;         /* Hub */
   newnodedes.BaudRate = (char) (nodedes.nodebaud / 300);       /* Baud Rate                */
   newnodedes.ModemType = 0;                     /* Don't know modem type    */
   newnodedes.NodeFlags = 0;                     /* Don't know any flags     */
   newnodedes.NodeFiller = 0;                    /* Filler should be zero    */

   /* Now get information from secondary file */
   ++recno;
   nodeoff = (long) recno *(long) sizeof (ext);  /* actual file offset        */

   strcpy (temp, net_info);                      /* take nodelist path        */
   strcat (temp, "NODELIST.EXT");                /* add in the file name      */
   newnodelist = 0;
   if ((stream = fopen (temp, "rb")) == NULL)    /* OK, let's open the file   */
      {
      return (1);
      }

   if (fseek (stream, nodeoff, SEEK_SET))        /* try to point at record    */
      {
      fclose (stream);
      return (1);
      }

   if (!fread (&ext, sizeof (ext), 1, stream))
      {
      fclose (stream);
      return (1);
      }
   fclose (stream);

   /*
    * Copy data from ext into newnodedes. 
    */

   strncpy (newnodedes.Password, ext.password, 8);      /* Password */

   newnodedes.NodeFlags = ext.flags1;            /* Nodelist flags */
   newnodelist = 1;                              /* We have all the
                                                  * information! */
   return (1);
}

int TBBSLookup (TBBSzone, TBBSnet, TBBSnode, have_boss_data)
int TBBSzone, TBBSnet, TBBSnode;                 /* zone, net and node */
int have_boss_data;                              /* BOSS data flag     */
{
   register struct _ndi far *nodeidx;            /* index file                */
/* int foundnet = 0; */                          /* 'found the net' flag      */
   int found = 0;                                /* 'found the node' flag     */
   int nodeoff = 0;                              /* offset into nodelist.sys  */
   char temp[80];                                /* where we build filenames  */
   unsigned int i;
   int stream;
   struct stat f;
   int current_zone = 0;
   int got;

   newnodedes.NetNumber = newnodedes.NodeNumber = 0;

   if (node_index == NULL)
      {
      temp[0] = '\0';                            /* "null-terminated string"  */
      strcpy (temp, net_info);                   /* take nodelist path        */
      strcat (temp, "NODELIST.IDX");             /* add in the file name      */
      if ((stream = open(temp, O_RDONLY)) == -1)
         {
         i = 0;                                  /* Need this later           */
         if (have_boss_data != 2)
            status_line ("!Unable to open Nodelist.Idx file");
         return (0);                             /* no file, no work to do    */
         }
      fstat (stream, &f);                        /* get file statistics       */
      i = (unsigned int) f.st_size;              /* size of index file,       */
      idx_size = i / sizeof (struct _ndi);       /* number of index entries   */
      node_index = get_size (i);
      if (node_index == NULL)
         {
         status_line ("!Unable to allocate memory for Nodelist Index");
         close (stream);
         return (0);
         }
      if (_dos_read (stream, node_index, i, &got) != 0)
         {
         status_line ("!Failed to read nodelist index into memory");
         close (stream);
         return (0);
         }
      close (stream);
      }
   nodeidx = (struct _ndi far *) node_index;

   if (TBBSzone < 0)
      {
      return (opus_next_zone ());
      }

   for (i = 1; i <= idx_size; nodeoff++, nodeidx++, i++)
      {
      if (nodeidx->node == -2)
	 current_zone = nodeidx->net;
	 
      if (TBBSzone > 0 && current_zone != TBBSzone)
            continue;

      if (nodeidx->net == TBBSnet)               /* if a match on net,        */
         {
/*       foundnet = 1;  */                       /* say we found the net      */
         if (((TBBSnode == 0) && (nodeidx->node <= 0))
             || (nodeidx->node == TBBSnode))     /* see if we found the node  */
            {
            found = 1;                           /* say we found it           */
            break;                               /* get out                   */
            }
         }

      /*
       * This code is good, but many people complained about not being able
       * to add nodes to nets by adding them at the end of the nodelist.  So,
       * I commented this stuff out for now. - Bob 
       */
/*      else*/
      /*      if (foundnet)*//* already past the net?     */
      /*         break;*//* Yes, we failed...         */
      }

   if (!found)
      return (0);

   found_zone = current_zone;                    /* Keep track of found zone */

   i = get_TBBS_info (nodeoff, TBBSnet, TBBSnode);

   return (i);
}
